using System;
using UnityEngine;
using Random = UnityEngine.Random;

namespace CheesyFX
{
    public class Shaker : MonoBehaviour
    {
        private Rigidbody RB;
        private int axis;
        private float timer;
        private float forceTimer;
        private float flip;
        private bool applyForceOnReturn = true;

        private Vector3 currentForce;
        private Vector3 currentTorque;
        private Vector3 targetForce;
        private Vector3 targetTorque;
        private float forceFactor;
        private float rotation;
        
        private float periodRatio = .5f;
        private float period = .15f;
        private float baseForceFactor = 300f;
        private float forceDuration = 1f;
        private float forceQuickness = 1f;
        private float torqueQuickness = 1f;

        private float rng;
        private int rngCounter;

        public Shaker Init(Rigidbody rb, int axis = 0)
        {
            RB = rb;
            this.axis = axis;
            if(RB.name.StartsWith("l")) baseForceFactor = -baseForceFactor;
            if (axis == 2)
            {
                baseForceFactor *= 3f;
            }
            return this;
        }

        private void Randomize()
        {
            forceQuickness = Random.Range(1f, 2f);
            forceFactor = Random.Range(baseForceFactor*.8f, baseForceFactor*1.5f);
            switch (axis)
            {
                case 0:
                    period = Random.Range(.15f, .25f);
                    periodRatio = Random.Range(.3f, .8f);
                    rotation = Random.Range(0f, 90f);
                    break;
                case 2:
                    period = Random.Range(.2f, .3f);
                    periodRatio = Random.Range(.2f, .7f);
                    rotation = Random.Range(0f, 45f);
                    break;
            }
        }

        private void SetTargets()
        {
            timer -= Time.fixedDeltaTime;
            forceTimer -= Time.fixedDeltaTime;
            if (Random.Range(0f, 1f) < .1f)
            {
                targetForce = Vector3.zero;
                return;
            }
            if (timer < 0.0f) {
                if ((flip > 0f && periodRatio != 1f) || periodRatio == 0f) {
                    if (applyForceOnReturn) {
                        flip = -1f;
                    } else {
                        flip = 0f;
                    }
                    timer = period * (1f - periodRatio);
                    forceTimer = forceDuration * period;
                } else {
                    flip = 1f;
                    timer = period * periodRatio;
                    forceTimer = forceDuration * period;
                }
                SetForce(flip);
            } else if (forceTimer < 0.0f) {
                SetForce(0f);
            }
        }
        
        // protected void Update() {
        //     
        // }
        
        void FixedUpdate() {
            try
            {
                rngCounter++;
                if (rngCounter > 30)
                {
                    Randomize();
                    rngCounter = 0;
                }
                SetTargets();
                // apply forces here
                float timeFactor = Time.fixedDeltaTime;
                currentForce = Vector3.Lerp(currentForce, targetForce, timeFactor * forceQuickness);
                // currentTorque = Vector3.Lerp(currentTorque, targetTorque, timeFactor * torqueQuickness);
                if (RB && (!SuperController.singleton || !SuperController.singleton.freezeAnimation)) {
                    RB.AddForce(currentForce, ForceMode.Force);
                    // RB.AddTorque(currentTorque, ForceMode.Force);
                }
            }
            catch (Exception e) {
                SuperController.LogError("Exception caught: " + e);
            }
        }
        
        protected void SetForce(float percent) {
            switch (axis)
            {
                case 0:
                    targetForce = percent * forceFactor * (Quaternion.AngleAxis(rotation, RB.transform.up) * RB.transform.right);
                    break;
                case 1:
                    targetForce = percent * forceFactor * RB.transform.up;
                    break;
                case 2:
                    targetForce = percent * forceFactor * (Quaternion.AngleAxis(rotation, RB.transform.right) * RB.transform.forward);
                    break;
            }
            // targetTorque = AxisToVector(_torqueAxis) * percent * torqueFactor;
        }

        // private void OnDestroy()
        // {
        //     
        // }
    }
}